#ifndef CBLAS_H
#define CBLAS_H

#include <stddef.h>
#include <stdint.h>
#include <complex.h>

#ifdef __cplusplus
extern "C" {
#endif  /* __cplusplus */

#define CBLAS_INDEX size_t

typedef int blasint;
typedef enum CBLAS_ORDER     {CblasRowMajor=101, CblasColMajor=102} CBLAS_ORDER;
typedef enum CBLAS_TRANSPOSE {CblasNoTrans=111, CblasTrans=112,
                              CblasConjTrans=113, CblasConjNoTrans=114} CBLAS_TRANSPOSE;
typedef enum CBLAS_UPLO      {CblasUpper=121, CblasLower=122} CBLAS_UPLO;
typedef enum CBLAS_DIAG      {CblasNonUnit=131, CblasUnit=132} CBLAS_DIAG;
typedef enum CBLAS_SIDE      {CblasLeft=141, CblasRight=142} CBLAS_SIDE;
typedef CBLAS_ORDER CBLAS_LAYOUT;

/* blas level1 */
void   cblas_srotg(float *a, float *b, float *c, float *s);
void   cblas_drotg(double *a, double *b, double *c, double *s);

void   cblas_srotmg(float *d1, float *d2, float *b1, float b2, float *P);
void   cblas_drotmg(double *d1, double *d2, double *b1, double b2, double *P);

void   cblas_srot(blasint n, float *x, blasint incx, float *y,
                  blasint incY, float c, float s);
void   cblas_drot(blasint n, double *x, blasint incx, double *y,
                  blasint incY, double c, double s);

void   cblas_sswap(blasint n, float *x, blasint incx, float *y, blasint incy);
void   cblas_dswap(blasint n, double *x, blasint incx, double *y, blasint incy);

void   cblas_srotm(blasint n, float *x, blasint incx, float *y,
                   blasint incy, float *P);
void   cblas_drotm(blasint n, double *x, blasint incx, double *y,
                   blasint incy, double *P);

void   cblas_scopy(blasint n, float *x, blasint incx, float *y, blasint incy);
void   cblas_dcopy(blasint n, double *x, blasint incx, double *y, blasint incy);

void   cblas_saxpy(blasint n, float alpha, float *x,
                   blasint incx, float *y, blasint incy);
void   cblas_daxpy(blasint n, double alpha, double *x,
                   blasint incx, double *y, blasint incy);

void   cblas_sscal(blasint n, float alpha, float *x, blasint incx);
void   cblas_dscal(blasint n, double alpha, double *x, blasint incx);

float  cblas_sdsdot(blasint n, float alpha, float *x, blasint incx,
                    float *y, blasint incy);
double cblas_dsdot(blasint n, float *x, blasint incx, float *y, blasint incy);

float  cblas_snrm2(blasint N, float *x, blasint incx);
double cblas_dnrm2(blasint N, double *x, blasint incx);

float  cblas_scnrm2(blasint N, void *x, blasint incx);
double cblas_dznrm2(blasint N, void *x, blasint incx);

float  cblas_sasum(blasint n, float *x, blasint incx);
double cblas_dasum(blasint n, double *x, blasint incx);

CBLAS_INDEX cblas_isamax(blasint n, float *x, blasint incx);
CBLAS_INDEX cblas_idamax(blasint n, double *x, blasint incx);

float  cblas_sdot(blasint n, float *x, blasint incx, float *y, blasint incy);
double cblas_ddot(blasint n, double *x, blasint incx, double *y, blasint incy);

void cblas_crotg(void *a, void *b, float *c, void *s);
void cblas_zrotg(void *a, void *b, double *c, void *s);

void cblas_csrot(blasint n, void *x, blasint incx, void *y, blasint incY,
                 float c, float s);
void cblas_zdrot(blasint n, void *x, blasint incx, void *y, blasint incY,
                 double c, double s);

void cblas_cswap(blasint n, void *x, blasint incx, void *y, blasint incy);
void cblas_zswap(blasint n, void *x, blasint incx, void *y, blasint incy);

void cblas_cscal(blasint N, void *alpha, void *X, blasint incX);
void cblas_zscal(blasint N, void *alpha, void *X, blasint incX);

void cblas_csscal(blasint N, float alpha, void *X, blasint incX);
void cblas_zdscal(blasint N, double alpha, void *X, blasint incX);

void cblas_ccopy(blasint n, void *x, blasint incx,
                 void *y, blasint incy);
void cblas_zcopy(blasint n, void *x, blasint incx,
                 void *y, blasint incy);

void cblas_caxpy(blasint n, void *alpha, void *x, blasint incx, void *y, blasint incy);
void cblas_zaxpy(blasint n, void *alpha, void *x, blasint incx, void *y, blasint incy);

float _Complex cblas_cdotu(blasint n, void *x, blasint incx, void  *y, blasint incy);
float _Complex cblas_cdotc(blasint n, void *x, blasint incx, void  *y, blasint incy);
double _Complex cblas_zdotu(blasint n, void *x, blasint incx, void *y, blasint incy);
double _Complex cblas_zdotc(blasint n, void *x, blasint incx, void *y, blasint incy);

float  cblas_scasum(blasint n, void *x, blasint incx);
double cblas_dzasum(blasint n, void *x, blasint incx);

CBLAS_INDEX cblas_icamax(blasint n, void *x, blasint incx);
CBLAS_INDEX cblas_izamax(blasint n, void *x, blasint incx);

/* blas level2 */
void cblas_dgemv(enum CBLAS_ORDER order,  enum CBLAS_TRANSPOSE trans,
                 blasint m, blasint n, double alpha,
                 double  *a, blasint lda,  double  *x,
                 blasint incx,  double beta,  double  *y,
                 blasint incy);
void cblas_dtrsv(enum CBLAS_ORDER order, enum CBLAS_UPLO Uplo,
                 enum CBLAS_TRANSPOSE TransA, enum CBLAS_DIAG Diag,
                 blasint N, double *A, blasint lda,
                 double *X, blasint incX);
void cblas_dger(enum CBLAS_ORDER order, blasint M, blasint N,
                double  alpha, double *X, blasint incX,
                double *Y, blasint incY, double *A, blasint lda);

/* blas level3 */
void cblas_dgemm(enum CBLAS_ORDER Order,  enum CBLAS_TRANSPOSE TransA,
                 enum CBLAS_TRANSPOSE TransB,  blasint M,  blasint N,
                 blasint K,  double alpha,  double *A,  blasint lda,
                 double *B,  blasint ldb,  double beta, double *C,  blasint ldc);

void cblas_dtrsm(enum CBLAS_ORDER Order, enum CBLAS_SIDE Side,
                 enum CBLAS_UPLO Uplo, enum CBLAS_TRANSPOSE TransA,
                 enum CBLAS_DIAG Diag, blasint M,
                 blasint N, double alpha, double *A,
                 blasint lda, double *B, blasint ldb);

/* blas like */
uint8_t  cblas_u8dot(blasint n,  uint8_t  *x,  blasint incx,  uint8_t  *y,  blasint incy);
int8_t   cblas_i8dot(blasint n,  int8_t   *x,  blasint incx,  int8_t   *y,  blasint incy);
int16_t  cblas_i16dot(blasint n, int16_t  *x,  blasint incx,  int16_t  *y,  blasint incy);
int32_t  cblas_i32dot(blasint n, int32_t  *x,  blasint incx,  int32_t  *y,  blasint incy);

#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif /* CBLAS_H */
